@@ -686,6 +686,24 @@ grl_media_set_date (GrlMedia *media, const gchar *date)
                        date);
 }
 
+/**
+  * grl_media_set_creation_date:
+  * @image: the media
+  * @creation_date: date when media was created
+  *
+  * Set the creation_date of the media
+  *
+  * Since: 0.1.12
+  */
+void
+grl_media_set_creation_date (GrlMedia *media,
+                             const gchar * creation_date)
+{
+  grl_data_set_string (GRL_DATA (media),
+                       GRL_METADATA_KEY_CREATION_DATE,
+                       creation_date);
+}
+
 /**
  * grl_media_set_mime:
  * @media: the media
@@ -1115,6 +1133,21 @@ grl_media_get_date (GrlMedia *media)
   return grl_data_get_string (GRL_DATA (media), GRL_METADATA_KEY_DATE);
 }
 
+/**
+ * grl_media_get_creation_date:
+ * @image: the media
+ *
+ * Returns: date when media was created
+ *
+ * Since: 0.1.12
+ */
+const gchar *
+grl_media_get_creation_date (GrlMedia *media)
+{
+  return grl_data_get_string (GRL_DATA (media),
+                              GRL_METADATA_KEY_CREATION_DATE);
+}
+
 /**
  * grl_media_get_mime:
  * @media: the media object
